/****************************************************************************
 *
 * omp_sections.c - Demostration of the OpenMP "sections" work sharing directive
 *
 * Last updated in 2017 by Moreno Marzolla <moreno.marzolla(at)unibo.it>
 *
 * To the extent possible under law, the author(s) have dedicated all 
 * copyright and related and neighboring rights to this software to the 
 * public domain worldwide. This software is distributed without any warranty.
 *
 * You should have received a copy of the CC0 Public Domain Dedication
 * along with this software. If not, see 
 * <http://creativecommons.org/publicdomain/zero/1.0/>. 
 *
 * ----------------------------------------------------------------------------
 *
 * Compile with:
 * gcc -fopenmp omp_sections.c -o omp_sections
 *
 * Run with:
 * OMP_NUM_THREADS=4 ./omp_sections
 *
 *****************************************************************************/
#include <stdio.h>
#define N 1000

int main(void)
{
    int i;
    float a[N], b[N], c[N], d[N];
    /* Some initializations */
    for (i=0; i < N; i++) {
	a[i] = i * 1.5;
	b[i] = i + 22.35;
    }
#pragma omp parallel shared(a,b,c,d) private(i)
    {
#pragma omp sections nowait
	{
#pragma omp section
	    for (i=0; i < N; i++)
		c[i] = a[i] + b[i];
#pragma omp section
	    for (i=0; i < N; i++)
		d[i] = a[i] * b[i];
	}  /* end of sections (no barrier here) */
    }  /* end of parallel section (barrier here) */

    for ( i=0; i<N; i++) {
	printf("%f %f %f %f\n", a[i], b[i], c[i], d[i]);
    }

    return 0;
}
